package nl.kallestruik.darena.arenas

import nl.kallestruik.darena.types.arena.ArenaLoadout
import nl.kallestruik.darena.types.arena.ArenaCheckpoint
import nl.kallestruik.darena.types.arena.ArenaPoints
import nl.kallestruik.darena.types.arena.ArenaSpawn
import nl.kallestruik.darena.types.arena.ArenaSpawnRule
import nl.kallestruik.darena.util.ConfigHelper
import java.io.File
import org.bukkit.configuration.InvalidConfigurationException
import org.bukkit.configuration.file.YamlConfiguration
import kotlin.collections.mutableMapOf

data class ArenaConfig(
    var name: String,
    var file: File,
    var spawns: MutableMap<String, ArenaSpawn> = mutableMapOf(),
    var loadouts: MutableMap<String, ArenaLoadout> = mutableMapOf(),
    var checkpoints: MutableMap<String, ArenaCheckpoint> = mutableMapOf(),
    var points: ArenaPoints = ArenaPoints(),
    var spawnRules: MutableMap<String, ArenaSpawnRule> = mutableMapOf()
) {
    fun save(toString: Boolean = false): String? {
        val config = YamlConfiguration()
        config.set("name", name)

        ConfigHelper.saveMap(ConfigHelper.getOrCreateSection(config, "spawns"), spawns)
        ConfigHelper.saveMap(ConfigHelper.getOrCreateSection(config, "loadouts"), loadouts)
        ConfigHelper.saveMap(ConfigHelper.getOrCreateSection(config, "checkpoints"), checkpoints)
        points.save(ConfigHelper.getOrCreateSection(config, "points"))
        ConfigHelper.saveMap(ConfigHelper.getOrCreateSection(config, "spawnRules"), spawnRules)

        if (toString) {
            return config.saveToString()
        }

        config.save(file)
        return null
    }

    companion object {
        @Throws(InvalidConfigurationException::class)
        fun load(file: File): ArenaConfig {
            val config = ConfigHelper.getOrCreateConfig(file, "template/arena.yml")
            if (!config.contains("name")) {
                throw InvalidConfigurationException("The arena configuration file '${file.name}' does not contain the required attribute 'name'")
            }

            val arenaConfig = ArenaConfig(config.getString("name")!!, file)

            if (config.contains("spawns")) {
                arenaConfig.spawns = ConfigHelper.loadMap(config.getConfigurationSection("spawns")!!, ArenaSpawn)
            }

            if (config.contains("loadouts")) {
                arenaConfig.loadouts = ConfigHelper.loadMap(config.getConfigurationSection("loadouts")!!, ArenaLoadout)
            }

            if (config.contains("checkpoints")) {
                arenaConfig.checkpoints = ConfigHelper.loadMap(config.getConfigurationSection("checkpoints")!!, ArenaCheckpoint)
            }

            if (config.contains("points")) {
                arenaConfig.points = ArenaPoints.load(config.getConfigurationSection("points")!!)
            }

            if (config.contains("spawnRules")) {
                arenaConfig.spawnRules = ConfigHelper.loadMap(config.getConfigurationSection("spawnRules")!!, ArenaSpawnRule)
            }

            return arenaConfig
        }
    }
}
