package nl.kallestruik.darena.arenas.world

import nl.kallestruik.darena.exceptions.ArenaWorldCreationException
import nl.kallestruik.darena.exceptions.ArenaWorldLoadException
import nl.kallestruik.darena.exceptions.ArenaWorldSaveException
import org.bukkit.Bukkit
import org.bukkit.World
import org.bukkit.plugin.java.JavaPlugin
import java.nio.file.Files
import java.nio.file.Path


class ArenaWorld(
    val name: String,
    val plugin: JavaPlugin
) {
    var world: World

    init {
        load()
        world = Bukkit.createWorld(ArenaWorldCreator(name))
            ?: throw ArenaWorldCreationException("Exception while creating bukkit world for arena \"$name\".")
    }
    @Throws(ArenaWorldLoadException::class)
    fun reset() {
        //TODO: Teleport everyone out.
        load()
    }

    @Throws(ArenaWorldSaveException::class)
    fun save() {
        try {
            val savePath = Path.of(plugin.dataFolder.path, "worlds", name)
            Files.walk(savePath).use { walk ->
                walk.sorted(Comparator.reverseOrder()).forEach { path ->
                    Files.delete(path)
                }
            }

            Files.walk(world.worldFolder.toPath()).use { walk ->
                walk.forEach { source ->
                    val destination = Path.of(
                        savePath.toString(), source.toString()
                            .substring(world.worldFolder.path.length)
                    )

                    Files.copy(source, destination)
                }
            }
        } catch (e: Exception) {
            throw ArenaWorldSaveException("There was an issue saving the world for arena \"$name\" to a safe location!", e)
        }
    }

    @Throws(ArenaWorldLoadException::class)
    private fun load() {
        try {
            val loadPath = Path.of(plugin.dataFolder.path, "worlds", name)
            Files.walk(world.worldFolder.toPath()).use { walk ->
                walk.sorted(Comparator.reverseOrder()).forEach { path ->
                    Files.delete(path)
                }
            }

            Files.walk(loadPath).use { walk ->
                walk.forEach { source ->
                    val destination = Path.of(
                        world.worldFolder.path, source.toString()
                            .substring(loadPath.toString().length)
                    )

                    Files.copy(source, destination)
                }
            }
        } catch (e: Exception) {
            throw ArenaWorldLoadException("There was an issue load the world for arena \"$name\"!", e)
        }
    }
}