package nl.kallestruik.darena.managers

import net.kyori.adventure.text.format.TextColor
import nl.kallestruik.darena.exceptions.TeamExistsException
import nl.kallestruik.darena.exceptions.TeamNotFoundException
import nl.kallestruik.darena.types.Team
import java.io.File
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class TeamManager(
    val teamsFile: File
) {
    private val teams: MutableMap<String, Team> = HashMap()
    private val teamMembers: MutableMap<Team, MutableList<UUID>> = HashMap()
    private val teamForPlayer: MutableMap<UUID, Team> = HashMap()

    fun load() {
        //TODO: Load teams from file
    }

    @Throws(TeamExistsException::class)
    fun createTeam(teamName: String) {
        if (teams.containsKey(teamName))
            throw TeamExistsException("The team $teamName already exists!")

        teams[teamName] = Team(teamName)
    }

    @Throws(TeamNotFoundException::class)
    fun setTeamColor(teamName: String, color: TextColor) {
        val team = getTeamByName(teamName)

        team.color = color
    }

    @Throws(TeamNotFoundException::class)
    fun addPlayerToTeam(teamName: String, uuid: UUID) {
        val team = getTeamByName(teamName)

        teamMembers.getOrPut(team) { ArrayList() }.add(uuid)
        teamForPlayer[uuid] = team
    }

    @Throws(TeamNotFoundException::class)
    fun getTeamForPlayer(uuid: UUID): Team {
        return teamForPlayer[uuid] ?: throw TeamNotFoundException("Player with uuid $uuid is not part of a team!")
    }

    @Throws(TeamNotFoundException::class)
    fun getTeamByName(teamName: String): Team {
        return teams[teamName] ?: throw TeamNotFoundException("Team with name $teamName does not exist!")
    }

    @Throws(TeamNotFoundException::class)
    fun getTeamMembers(teamName: String): List<UUID> {
        val team = getTeamByName(teamName)
        return teamMembers[team] ?: return emptyList()
    }
}